/****************************************************
AboutSQLServer.com blog
Written by Dmitri Korotkevitch

Optimizing SQL Server Spatial Queries with Bounding Box
2013-09-03
*****************************************************/

if exists
(
	select * 
	from 
		sys.tables t join sys.schemas s on
			t.schema_id = s.schema_id
	where 
		t.name = 'Locations' and s.name = 'dbo'
) 
	drop table dbo.Locations
go

if exists
(
	select * 
	from 
		sys.tables t join sys.schemas s on
			t.schema_id = s.schema_id
	where 
		t.name = 'LocationsGeo' and s.name = 'dbo'
) 
	drop table dbo.LocationsGeo
go

if exists
(
	select * 
	from 
		sys.tables t join sys.schemas s on
			t.schema_id = s.schema_id
	where 
		t.name = 'LocationsGeoIndexed' and s.name = 'dbo'
) 
	drop table dbo.LocationsGeoIndexed
go


if exists
(
	select * 
	from 
		sys.tables t join sys.schemas s on
			t.schema_id = s.schema_id
	where 
		t.name = 'LocationsGeo2' and s.name = 'dbo'
) 
	drop table dbo.LocationsGeo2
go

if object_id(N'[dbo].[CalcDistanceCLR]') is not null
	drop function [dbo].CalcDistanceCLR
go

if object_id(N'[dbo].CalcCircleBoundingBox') is not null
	drop function [dbo].CalcCircleBoundingBox
go

if exists(select * from sys.assemblies where name = 'CLRTest')
	drop assembly CLRTest
go

create assembly [CLRTest]
authorization [dbo]
from 'c:\SQL\CLRTest\CLRTest.dll'
with permission_set = safe
GO

create function [dbo].[CalcDistanceCLR]
(
 @fromLat float(53),
 @fromLon float(53),
 @toLat float(53),
 @toLon float(53)
)
returns float(53)
as external name 
	[CLRTest].[UserDefinedFunctions].[CalcDistanceCLR];
go

create table dbo.Locations
(
	Id int not null identity(1,1),
	Latitude decimal(9,6) not null,
	Longitude decimal(9,6) not null,
	primary key(Id)
)
go

create table dbo.LocationsGeo
(
	Id int not null identity(1,1),
	Location geography not null,
	primary key(Id)
)
go  

create table dbo.LocationsGeoIndexed
(
	Id int not null identity(1,1),
	Location geography not null,
	primary key(Id)
)
go

;with Latitudes(Lat)
as
(
	select convert(float,40.0)
	
	union all
	
	select convert(float,Lat + 0.01)
	from Latitudes
	where Lat < convert(float,48.0)
)
,Longitudes(Lon)
as
(
	select convert(float,-120.0)
	
	union all
	
	select Lon - 0.01
	from Longitudes
	where Lon > -123
)
insert into dbo.Locations(Latitude, Longitude)
	select Latitudes.Lat, Longitudes.Lon
	from Latitudes cross join Longitudes
option (maxrecursion 0)
go

insert into dbo.LocationsGeo(Location)
	select geography::Point(Latitude, Longitude, 4326)
	from dbo.Locations
go

insert into dbo.LocationsGeoIndexed(Location)
	select Location
	from dbo.LocationsGeo
go

create spatial index Idx_LocationsGeoIndexed_Spatial
on dbo.LocationsGeoIndexed(Location)
go

                                
-- Performance test
set statistics time on
go

declare
	@Lat decimal(9,6) = 47.620309
	,@Lon decimal(9,6) = -122.349563

declare
	@g geography = geography::Point(@Lat,@Lon,4326)

select ID
from dbo.Locations
where dbo.CalcDistanceCLR(Latitude, Longitude, @Lat, @Lon) < 1609

select ID
from dbo.LocationsGeo
where Location.STDistance(@g) < 1609

select ID
from dbo.LocationsGeoIndexed
where Location.STDistance(@g) < 1609
go

create table dbo.LocationsGeo2
(
	CompanyId int not null,
	Id int not null identity(1,1),
	Location geography not null,
	constraint PK_LocationsGeo2
	primary key clustered(CompanyId,Id)
)
go  

-- 50 companies, 241,402 rows each
;with Companies(CID)
as
(
	select 1

	union all 

	select CID + 1 from Companies where CID < 50
)
insert into dbo.LocationsGeo2(CompanyId,Location)
	select c.CID, l.Location
	from dbo.LocationsGeo l cross join Companies c;

create spatial index Idx_LocationsGeo2_Spatial
on dbo.LocationsGeo2(Location)
go

declare
	@Lat decimal(9,6) = 47.620309
	,@Lon decimal(9,6) = -122.349563
	,@CompanyId int = 15

declare
	@g geography = geography::Point(@Lat,@Lon,4326)
	
select count(*)
from dbo.LocationsGeo2 
where 
	Location.STDistance(@g) < 1609 and 
	CompanyId = @CompanyId
	
select count(*)
from dbo.LocationsGeo2 
	with (index=Idx_LocationsGeo2_Spatial)
where 
	Location.STDistance(@g) < 1609 and 
	CompanyId = @CompanyId
go

-- Adding bounding box
create function [dbo].[CalcCircleBoundingBox]
( @lat float(53), @lon float(53), @distance int)
returns table
(
	 [MinLat] float(53) null,
	 [MaxLat] float(53) null,
	 [MinLon] float(53) null,
	 [MaxLon] float(53) null
)
as external name 
	[CLRTest].[UserDefinedFunctions].[CalcCircleBoundingBox]
go

alter table dbo.LocationsGeo2 add MinLat decimal(9,6)
alter table dbo.LocationsGeo2 add MaxLat decimal(9,6)
alter table dbo.LocationsGeo2 add MinLon decimal(9,6)
alter table dbo.LocationsGeo2 add MaxLon decimal(9,6)
go

update t
set
	t.MinLat = b.MinLat
	,t.MinLon = b.MinLon
	,t.MaxLat = b.MaxLat
	,t.MaxLon = b.MaxLon
from
	dbo.LocationsGeo2 t cross apply
		dbo.[CalcCircleBoundingBox]
		(
			t.Location.Lat,
			t.Location.Long,
			1609
		) b
go

create index IDX_LocationsGeo2_BoundingBox
on dbo.LocationsGeo2(CompanyId, MinLon, MaxLon)
include (MinLat, MaxLat)
go


declare
	@Lat decimal(9,6) = 47.620309
	,@Lon decimal(9,6) = -122.349563
	,@CompanyId int = 15
	
declare
	@g geography = geography::Point(@Lat,@Lon,4326)
	
select ID
from dbo.LocationsGeo2 
where 
	Location.STDistance(@g) < 1609 and 
	CompanyId = @CompanyId and
	@Lat between MinLat and MaxLat and 
	@Lon between MinLon and MaxLon
go
